%% Initializing Simulation variables

% The initial position and velocity of the system
Sim.Eta_init        = [1.0 1.0 100 0 0 0]';
Sim.Nu_init         = [0 0 0 0 0 0]';
Sim.Ship_position   = [1 1 100]; %[N E lenght of wire]

% distance from Xmas to ROV in Xmas(system) body
Sim.r_x = [-2 0 0 0 0 0]';
% Translation matrix for system
Sim.H = Hmtrx(Sim.r_x);
Sim.J = eulerang(Sim.r_x(4),Sim.r_x(5),Sim.r_x(6));

Sim.T_save = 0.2;
Sim.T_sim = 1000;
Sim.T_step = 0.02;
Sim.meas_update = 0.2; %5 Hz
Sim.SensNoiseEnabled = 1;
Sim.PPM_Noise_SampleTime = 0.02;


Sim.g = 9.81;
Sim.rho = 1025;
Sim.p_0 = 101325;
Sim.p_w = 1230;

Controller.Kp = diag([50 50 300])*10;
Controller.Kd = diag([300 300 1000])*10;
Controller.Ki = diag([1.5 1.5 2.5])*30;

Controller.Kp = diag([2000 2000 2000]);
Controller.Kd = diag([8000 8000 38500]);
Controller.Ki = diag([350 350 150])*0.5;

% DEBUGGING OPTIONS for ROV, all set to 1 for normal simulations
Sim.C_rb				= 1;			% 1 for including C_rb (rigid-body coriolis-centripetal forces), 0 for not
Sim.C_a					= 1;			% 1 for including C_a (added mass coriolis-centripetal forces), 0 for not
Sim.D_l					= 1;			% 1 for including D_l (linear damping forces), 0 for not
Sim.D_nl				= 1;			% 1 for including D_nl (non-linear damping forces), 0 for not
Sim.G					= 0;            % 1 for including G (restoring forces), 0 for not
Sim.Fcable              = 1;            % 1 for including the force from cable drag, 0 for not.
Sim.Observer            = 0;            % 1 for using the observar as input to the controller, 0 for not.

%% Current
% This is just for surface velocity. Define the current profile including depth below as Par.Current.Profile
% NB! DEFINE THE MEAN CURRENT DIRECTION AS FUNCTION OF TIME TO FACILITATE
% FAST CHANGES
Par.Current.Waves       = 0;            % 1 for waves , 0 to turn off waves.
Par.Current.vel 		= 0;          % Current velocity (m/s)
Par.Current.dir			= 45*pi/180;    % Current direction (rad)
Par.Current.var_vel		= 0;			% 1 for variation of velocity, 0 for constant
Par.Current.var_dir		= 0;			% 1 for variation of direction, 0 for constant
Par.Current.mju_vel		= 0;		    % Gauss Markov process constant for varying velocity. 0 for random walk.
Par.Current.sat_vel		= 0.2;			% Max variation in current velocity: velocity = [vel - sat_vel, vel + sat_vel]
Par.Current.wn_t_vel 	= Sim.T_step;		% Velocity white noise sample time
Par.Current.wn_p_vel 	= 0.000001;		% Velocity white noise power
Par.Current.wn_s_vel 	= 1234;			% Velocity white noise seed
Par.Current.mju_dir		= 0;			% Gauss Markov process constant for varying direction. 0 for random walk.
Par.Current.sat_dir		= 5*pi/180;		% Max variation in current direction: direction = [dir - sat_dir, dir + sat_dir]
Par.Current.wn_t_dir 	= Sim.T_step;		% Direction white noise sample time
Par.Current.wn_p_dir 	= 0.000001;		% Direction white noise power
Par.Current.wn_s_dir 	= 4321;			% Direction white noise seed

% Define current profile [xvel, yvel, depth]. If only using surface component, it is fully defined by the surface velocity and direction
Par.Current.Profile = ...
	[Par.Current.vel*[cos(Par.Current.dir), sin(Par.Current.dir)], 0;
	 Par.Current.vel*[cos(Par.Current.dir), sin(Par.Current.dir)], 5];



%% Initializing Xmas Tree parameters
Xmas.Scale = 1;

Xmas.D_Cable = 0.1*Xmas.Scale;

%  Inititial position and velocity of Xmas tree
Xmas.Eta_init   = Sim.Eta_init;
Xmas.Nu_init    = Sim.Nu_init;

Xmas.Mass = 36000 * Xmas.Scale^3; %kg
Xmas.Length = 3.6 * Xmas.Scale; %m
Xmas.Width = 2.586 * Xmas.Scale; %m
Xmas.Height = 2.808 * Xmas.Scale; %m
V_b = Xmas.Height * Xmas.Width * Xmas.Length; %m^3, Submerged volume of the block
Xmas.Volume = 0.6 * V_b; %m^3, Submerged volume, assumed a block-coefficient of 0.6


Xmas.I_mtrx = [ (1/12)*Xmas.Mass*(Xmas.Width^2 + Xmas.Height^2) 0 0;
                    0 (1/12)*Xmas.Mass*(Xmas.Height^2 + Xmas.Length^2) 0;
                    0 0 (1/12)*Xmas.Mass*(Xmas.Width^2 + Xmas.Length^2)];
        
Xmas.M_rb = [   Xmas.Mass*eye(3) zeros(3);
                    zeros(3) Xmas.I_mtrx];
                
Xmas.M_a  = diag([2*0.68* Sim.rho * Xmas.Volume
                      2*0.68* Sim.rho * Xmas.Volume
                      2*0.68* Sim.rho * Xmas.Volume
                      0.8*0.234*Sim.rho*pi*Xmas.Height^4
                      0.8*0.19*Sim.rho*pi*Xmas.Length^4
                      0.8*0.19*Sim.rho*pi*Xmas.Length^4]);

Xmas.M_tot = Xmas.M_a + Xmas.M_rb;

k_tot = vertical_stiffness(Xmas);
Cc = 2 * Xmas.M_tot(3,3) * sqrt(k_tot/Xmas.M_tot(3,3));
        
Xmas.D_nl = diag([  0.5*0.6285*1.2*Xmas.Width*Xmas.Height*Sim.rho 
                    0.5*0.6285*1.8*Xmas.Length*Xmas.Height*Sim.rho 
                    0.5*0.6285*1.8*Xmas.Width*Xmas.Length*Sim.rho 
                    0.25*Sim.rho*Xmas.Length*Xmas.Width*(0.5*Xmas.Height)^4 
                    0.25*Sim.rho*Xmas.Length*Xmas.Width*(0.5*Xmas.Length)^4 
                    0.25*Sim.rho*Xmas.Length*Xmas.Height*(0.5*Xmas.Length)^4]);
                
Xmas.D_l = diag([   0.5*0.1433*1.2*Xmas.Width*Xmas.Height*Sim.rho
                    0.5*0.1433*1.8*Xmas.Length*Xmas.Height*Sim.rho 
                    Cc*0.01 
                    Xmas.D_nl(4,4); %% OBS OBS
                    Xmas.D_nl(5,5)*0.3; %% OBS OBS
                    Xmas.D_nl(6,6)*0.3;]); %% OBS OBS

Xmas.W = Xmas.Mass * Sim.g;
Xmas.B = Xmas.Volume * Sim.rho * Sim.g;
Xmas.W_submerged = Xmas.W - Xmas.B;

Xmas.Eta_init(3) = Xmas.Eta_init(3) + Xmas.W_submerged/k_tot;

Xmas.L_top_pipe = 1.18 * Xmas.Scale;
Xmas.arm = Xmas.Height/2 + Xmas.L_top_pipe; 

clear V_b k_tot Cc
%% Initializing ROV parameters

% Initial position and velocity of ROV
ROV.Eta_init    = Sim.Eta_init + Sim.r_x;
ROV.Nu_init     = Sim.H * Sim.Nu_init;
% distance from ROV to Xmas tree in ROV Frame
Sim.r_ROV =-1* eulerang(ROV.Eta_init(4),ROV.Eta_init(5),ROV.Eta_init(6))'*eulerang(Xmas.Eta_init(4),Xmas.Eta_init(5),Xmas.Eta_init(6))*Sim.r_x;

% Physical sizes
ROV.Length      = 2.5;
ROV.Breadth     = 1.5;
ROV.Height      = 1.6;
ROV.Mass        = 3700;
ROV.W           = ROV.Mass*Sim.g;
ROV.B           = ROV.W ;%+ 5;

ROV.r_cg        = [0 0 0];
ROV.r_cb        = [0 0 -0.3872];

% Moments of inertia
I = [    525.39 1.44 33.41;
         1.44 794.20 2.60;
         33.41 2.60 691.23];
I = I*(3700/1862.87);

ROV.M_rb_CG = [ROV.Mass*eye(3) zeros(3); zeros(3) I];

ROV.M_rb = Hmtrx(ROV.r_cg)'*ROV.M_rb_CG*Hmtrx(ROV.r_cg);
ROV.M_a = [  779.79 -6.8773 -103.32 8.5426 -165.54 -7.8033;
             -6.8773 1222 51.29 409.44 -5.8488 62.726;
             -103.32 51.29 3659.9 6.1112 -386.42 10.774;
             8.5426 409.44 6.1112 534.9 -10.027 21.019;
             -165.54 -5.8488 -386.42 -10.027 842.69 -1.1162;
             -7.8033 62.726 10.775 21.019 -1.1162 224.32];

ROV.M_tot = ROV.M_rb + ROV.M_a;

% Damping terms
ROV.D_nl    = diag([748.22 992.53 1821.01 672 774.44 523.27]);
ROV.D_l     = diag([74.82 69.48 728.40 268.8 309.77 105.00]);
ROV.D_l_exp = [-2 -8 -1]'; %exp. decay rate for [u1 (u1>0) u2 u1 (u1<0)]
%ROV.D_l_cpm = diag([0.1*292 0.07*584 64 16 30 0.2]);

% Cable drag forces
ROV.Cable.d     = 0.02;
ROV.Cable.cd    = 1.2;
ROV.Cable.const = 0.5*Sim.rho*ROV.Cable.d*ROV.Cable.cd;
ROV.Cable.roll  = 0.9317;
ROV.Cable.pitch = ROV.Cable.roll;

% Thrust allocation
ROV.Thrust.Tconfig = [  0 0 0.9063 0.9063;
                        1 0 -0.4226 0.4226;
                        0 1 0 0;
                        -0.2373 0 -0.4113 0.4113;
                        0 -0.2862 0.2179 0.2179;
                        0.4878 0 0.7639 -0.7639];
        
ROV.Thrust.pseudoT=(ROV.Thrust.Tconfig/(ROV.Thrust.Tconfig'*ROV.Thrust.Tconfig))';

A = 0.00029112*10; % Times ten to get more power
ROV.Thrust.KT = eye(4)*A*1;
ROV.Thrust.KT(2,2)= ROV.Thrust.KT(2,2)*3; % Three vertical thrusters
ROV.Thrust.KT_inv=inv(ROV.Thrust.KT);
ROV.Thrust.n_max=1500;

ROV.Thrust.sat_tau  = [11873 6550.2 19651 0 0 13202]'; 

clear I A;
%% Finding system parameters for the rigid body system

System.B = Xmas.B + ROV.B;
System.W = Xmas.W + ROV.B;

%Damping matrix
System.D_l = Xmas.D_l + Sim.H'*Sim.J*ROV.D_l/Sim.J*Sim.H;

%Rigid body mass
System.M_rb = Xmas.M_rb + Sim.H'*Sim.J*ROV.M_rb/Sim.J*Sim.H;

%Added mass
System.M_a = Xmas.M_a + Sim.H'*Sim.J*ROV.M_a/Sim.J*Sim.H;

% Total Mass matrix specifeid in Xmas tree CO
System.M_tot = Xmas.M_tot + Sim.H'*Sim.J*ROV.M_tot/Sim.J*Sim.H; 
System.M_tot_inv = inv(System.M_tot);

%% Inititalizarion of reference filter
Delta = eye(6); %Critical damping
cutoff = 0.02;

Omega = [cutoff 0 0 0 0 0;       %cut off frequencies in 6 DOFs
         0 cutoff 0 0 0 0;
         0 0 cutoff 0 0 0;
         0 0 0 0.07 0 0;
         0 0 0 0 0.07 0;
         0 0 0 0 0 0.01];
     
nu_max=[1.03 0.67 0.62 0  0 1.05]';
nu_min=-nu_max*0.8;
nu_dot_max=[0.5 0.5 0.4 1 1 1]';
nu_dot_min = -nu_dot_max*0.8;

Observer_System=System;
wave_height = 8;
wave_freq = (2*pi)/10;

sim_time = 1400;
time1 = 100;
step1 = [0 0 0 0 0 90*pi/180]';
time2 = 500;
step2 = [1.5 0 0 0 0 0]';
time3 = 900;
step3 = [0 1.5 0 0 0 0]';
time4 = 900;
step4 = [0 0 0 0 0 0]';
time5 = 900;
step5 = [0 0 0 0 0 0]';


