%% Initializing Simulation variables

% The initial position and velocity of the system
Sim.Eta_init        = [0 0 400 0 0 0]';
Sim.Nu_init         = [0 0 0 0 0 0]';
Sim.Ship_position   = [1 1 400]; %[N E lenght of wire]

% distance from Xmas to ROV in Xmas(system) body
Sim.r_x = [-1.8 0 0 0 0 0]';
% Translation matrix for system
Sim.H = Hmtrx(Sim.r_x);
Sim.J = eulerang(Sim.r_x(4),Sim.r_x(5),Sim.r_x(6));

Sim.T_save = 0.2;
Sim.T_sim = 1000;
Sim.T_step = 0.02;
Sim.meas_update = 0.2; %5 Hz
Sim.SensNoiseEnabled = 1;
Sim.PPM_Noise_SampleTime = 1;


Sim.g = 9.81;
Sim.rho = 1025;
Sim.p_0 = 101325;
Sim.p_w = 1230;

% DEBUGGING OPTIONS for ROV, all set to 1 for normal simulations
Sim.C_rb				= 1;			% 1 for including C_rb (rigid-body coriolis-centripetal forces), 0 for not
Sim.C_a					= 1;			% 1 for including C_a (added mass coriolis-centripetal forces), 0 for not
Sim.D_l					= 1;			% 1 for including D_l (linear damping forces), 0 for not
Sim.D_nl				= 1;			% 1 for including D_nl (non-linear damping forces), 0 for not
Sim.G					= 0;            % 1 for including G (restoring forces), 0 for not
Sim.Fcable              = 1;            % 1 for including the force from cable drag, 0 for not.
Sim.Observer            = 0;

%% Current
% This is just for surface velocity. Define the current profile including depth below as Par.Current.Profile
% NB! DEFINE THE MEAN CURRENT DIRECTION AS FUNCTION OF TIME TO FACILITATE
% FAST CHANGES
Par.Current.Waves       = 0;            % 1 for waves , 0 to turn off waves.
Par.Current.vel 		= -0.1;          % Current velocity (m/s)
Par.Current.dir			= 45*pi/180;    % Current direction (rad)
Par.Current.var_vel		= 0;			% 1 for variation of velocity, 0 for constant
Par.Current.var_dir		= 0;			% 1 for variation of direction, 0 for constant
Par.Current.mju_vel		= 0;		    % Gauss Markov process constant for varying velocity. 0 for random walk.
Par.Current.sat_vel		= 0.2;			% Max variation in current velocity: velocity = [vel - sat_vel, vel + sat_vel]
Par.Current.wn_t_vel 	= Sim.T_step;		% Velocity white noise sample time
Par.Current.wn_p_vel 	= 0.000001;		% Velocity white noise power
Par.Current.wn_s_vel 	= 1234;			% Velocity white noise seed
Par.Current.mju_dir		= 0;			% Gauss Markov process constant for varying direction. 0 for random walk.
Par.Current.sat_dir		= 5*pi/180;		% Max variation in current direction: direction = [dir - sat_dir, dir + sat_dir]
Par.Current.wn_t_dir 	= Sim.T_step;		% Direction white noise sample time
Par.Current.wn_p_dir 	= 0.000001;		% Direction white noise power
Par.Current.wn_s_dir 	= 4321;			% Direction white noise seed

% Define current profile [xvel, yvel, depth]. If only using surface component, it is fully defined by the surface velocity and direction
Par.Current.Profile = ...
	[Par.Current.vel*[cos(Par.Current.dir), sin(Par.Current.dir)], 0;
	 Par.Current.vel*[cos(Par.Current.dir), sin(Par.Current.dir)], 5];



%% Initializing Xmas Tree parameters
if(~exist( 'Xmas', 'var' ))
    Xmas.Scale = 1/3.5;
end


Xmas.D_Cable = 0.1*Xmas.Scale;

%  Inititial position and velocity of Xmas tree
Xmas.Eta_init   = Sim.Eta_init;
Xmas.Nu_init    = Sim.Nu_init;

Xmas.Mass = 880; %kg
Xmas.Length = 1; %m
Xmas.Width = 1.5; %m
Xmas.Height = 1.2; %m
Xmas.Volume = 0.0696 + 0.289; %m^3, Submerged volume, assumed a block-coefficient of 0.6
Xmas.rho = 7800; %kg/m^3

I_pipe_z = 0.5*pi*Xmas.rho*Xmas.Height*(0.177^4 - 0.16^4);
M_pipe = pi*Xmas.rho*Xmas.Height*(0.177^2 - 0.16^2);
I_top_z = (1/12)*(Xmas.Width*Xmas.Length*0.01*Xmas.rho)*(Xmas.Width^2+Xmas.Length^2);
M_top = (Xmas.Width*Xmas.Length*0.01*Xmas.rho);
I_back_z = (1/12) * (Xmas.Width*Xmas.Height*0.01*Xmas.rho)*(Xmas.Width^2 + 0.01^2);
M_back = (Xmas.Width*Xmas.Height*0.01*Xmas.rho);

I_z_pipes = 2*(0.5^2*M_pipe) + 0.25^2*M_pipe + 3 * I_pipe_z;
I_z_back = 0.4^2*M_back + I_back_z;

I_z = I_z_pipes + I_z_back + 2 * I_top_z;


Xmas.I_mtrx = [ (1/12)*Xmas.Mass*(Xmas.Width^2 + Xmas.Height^2) 0 0;
                    0 (1/12)*Xmas.Mass*(Xmas.Height^2 + Xmas.Length^2) 0;
                    0 0 I_z];
        
Xmas.M_rb = [   Xmas.Mass*eye(3) zeros(3);
                    zeros(3) Xmas.I_mtrx];
                
Xmas.M_a  = diag([2*0.68* Sim.rho * Xmas.Volume
                      2*0.68* Sim.rho * Xmas.Volume
                      2*0.68* Sim.rho * Xmas.Volume
                      0.8*0.19*Sim.rho*pi*Xmas.Height^4
                      0.8*0.19*Sim.rho*pi*Xmas.Length^4
                      0.8*0.19*Sim.rho*pi*Xmas.Length^4]);

Xmas.M_tot = Xmas.M_a + Xmas.M_rb;

k_tot = vertical_stiffness(Xmas);
Cc = 2 * Xmas.M_tot(3,3) * sqrt(k_tot/Xmas.M_tot(3,3));
        
Xmas.D_nl = diag([  0.5*0.6285*1.2*Xmas.Width*Xmas.Height*Sim.rho 
                    0.5*0.6285*1.8*(3*0.177)*Xmas.Height*Sim.rho 
                    0.5*0.6285*1.8*Xmas.Width*Xmas.Length*Sim.rho 
                    0.25*Sim.rho*Xmas.Length*Xmas.Width*(0.5*Xmas.Height)^4 
                    0.25*Sim.rho*Xmas.Length*Xmas.Width*(0.5*Xmas.Length)^4 
                    0.25*Sim.rho*Xmas.Length*Xmas.Height*(0.5*Xmas.Length)^4]);
                
Xmas.D_l = diag([   0.5*0.1433*1.2*Xmas.Width*Xmas.Height*Sim.rho
                    0.5*0.1433*1.8*Xmas.Length*Xmas.Height*Sim.rho 
                    Cc*0.01 
                    0%Xmas.D_nl(4,4); %% OBS OBS
                    0%Xmas.D_nl(5,5)*0.3; %% OBS OBS
                    0]);%Xmas.D_nl(6,6)*0.3;]); %% OBS OBS
Xmas.D_nl(1,2) = Xmas.D_nl(1,1)*-1;

Xmas.W = Xmas.Mass * Sim.g;
Xmas.B = Xmas.Volume * Sim.rho * Sim.g;
Xmas.W_submerged = Xmas.W - Xmas.B;

Xmas.Eta_init(3) = Xmas.Eta_init(3) + Xmas.W_submerged/k_tot;

Xmas.L_top_pipe = 1.18 * Xmas.Scale;
Xmas.arm = Xmas.Height/2 + Xmas.L_top_pipe; 

clear V_b k_tot Cc
%% Initializing ROV parameters

% Initial position and velocity of ROV
ROV.Eta_init    = Sim.Eta_init + Sim.r_x;
ROV.Nu_init     = Sim.H * Sim.Nu_init;
% distance from ROV to Xmas tree in ROV Frame
Sim.r_ROV =-1* eulerang(ROV.Eta_init(4),ROV.Eta_init(5),ROV.Eta_init(6))'*eulerang(Xmas.Eta_init(4),Xmas.Eta_init(5),Xmas.Eta_init(6))*Sim.r_x;

% Physical sizes
ROV.Length      = 1.44;
ROV.Breadth     = 0.82;
ROV.Height      = 0.80;
ROV.Mass        = 460;
ROV.W           = ROV.Mass*Sim.g;
ROV.B           = ROV.W ;%+ 5;

ROV.r_cg        = [0 0 0.12];
ROV.r_cb        = [0 0 -0.15];
ROV.r_current   = [0 0 0];

% Moments of inertia
i_x		= ROV.Mass/12*(ROV.Breadth^2+ROV.Length^2);     
i_y		= ROV.Mass/12*(ROV.Length^2+ROV.Height^2);     
i_z		= ROV.Mass/12*(ROV.Breadth^2+ROV.Height^2);    
i_xy	= 0;                                
i_xz 	= 0;
i_yz 	= 0;

I0      = [ i_x  -i_xy -i_xz
            -i_xz  i_y  -i_yz
            -i_xz -i_yz  i_z ];

ROV.M_rb_CG = [ ROV.Mass * eye(3) zeros(3);
                zeros(3) I0];

ROV.M_rb = Hmtrx(ROV.r_cg)'*ROV.M_rb_CG*Hmtrx(ROV.r_cg);
ROV.M_a = diag([293 302 326 52 52 57]); 
%ROV.M_a(2,6)= 6; ROV.M_a(6,2)= 6;

ROV.M_tot = ROV.M_rb + ROV.M_a;

% Damping terms
ROV.D_nl    = diag([292 584 635 84 148 1]);
ROV.D_l     = diag([0.8*292 0.5*584 64 16 30 0.2]);
ROV.D_l_exp = [-2 -8 -1]'; %exp. decay rate for [u1 (u1>0) u2 u1 (u1<0)]
ROV.D_l_cpm = diag([0.1*292 0.07*584 64 16 30 0.2]);

% Cable drag forces
ROV.Cable.d     = 0.022;
ROV.Cable.cd    = 1.2;
ROV.Cable.const = 0.5*Sim.rho*ROV.Cable.d*ROV.Cable.cd;
ROV.Cable.roll  = 0; %ROV.Height/2 + ROV.r_cg(3);
ROV.Cable.pitch = 0; %ROV.Cable.roll;

% Thrust allocation
ROV.Thrust.sat_tau  = [478 256 389 0 0 151]'; %Maximum forces and moments. Values based on bollard pull
    
    % Thruster positions
    alpha   =10*pi/180; %Fixed angle
    rx_stb  =0.57;
    rx_port =rx_stb;
    rx_lat  =0.166;
    ry_stb  =0.24; 
    ry_port =ry_stb;

ROV.Thrust.Tconfig  = [ cos(alpha) cos(alpha) 0 0 0;
                        sin(alpha) -sin(alpha) 1 0 0;
                        0 0 0 1 1;
                        (-sin(alpha)*rx_stb-cos(alpha)*ry_stb) (+sin(alpha)*rx_port+cos(alpha)*ry_port) rx_lat 0 0];
ROV.Thrust.pseudoT  = ROV.Thrust.Tconfig'/(ROV.Thrust.Tconfig*ROV.Thrust.Tconfig');

ROV.Thrust.D_uni    = 0.22;
ROV.Thrust.D_lat    = 0.19;
ROV.Thrust.KT0fw    = 0.24;
ROV.Thrust.KT0b     = 0.15;

ROV.Thrust.thrloss  = [0.72 0.53 0.58 1.04]; %[fw b d lat]

ROV.Thrust.k_fw     =Sim.rho*ROV.Thrust.D_uni^4*ROV.Thrust.KT0fw*ROV.Thrust.thrloss(1);
ROV.Thrust.k_b      =Sim.rho*ROV.Thrust.D_uni^4*ROV.Thrust.KT0b*ROV.Thrust.thrloss(2);
ROV.Thrust.k_lat    =Sim.rho*ROV.Thrust.D_lat^4*ROV.Thrust.KT0fw*ROV.Thrust.thrloss(4);
ROV.Thrust.k_down   =Sim.rho*ROV.Thrust.D_uni^4*ROV.Thrust.KT0fw*ROV.Thrust.thrloss(3);
ROV.Thrust.k_up     =Sim.rho*ROV.Thrust.D_uni^4*ROV.Thrust.KT0b*ROV.Thrust.thrloss(3);
ROV.Thrust.n_max    =1450/60;

ROV.Thrust.Recursive=1; % Use recursive thrust allocation (1=yes,0=no) 
init_rec_thrust;

clear   i_x i_y i_z i_xy i_xz i_yz I0 ... 
        alpha rx_stb rx_port rx_lat ry_stb ry_port;
%% Finding system parameters for the rigid body system

%Damping matrix
System.D_l = Xmas.D_l + Sim.H'*Sim.J*ROV.D_l/Sim.J*Sim.H;

%Rigid body mass
System.M_rb = Xmas.M_rb + Sim.H'*Sim.J*ROV.M_rb/Sim.J*Sim.H;

%Added mass
System.M_a = Xmas.M_a + Sim.H'*Sim.J*ROV.M_a/Sim.J*Sim.H;

%Weight and bouyancy;
System.W = Xmas.W + ROV.W;
System.B = Xmas.B + ROV.B;

% Total Mass matrix specifeid in Xmas tree CO
System.M_tot = Xmas.M_tot + Sim.H'*Sim.J*ROV.M_tot/Sim.J*Sim.H; 
System.M_tot_inv = inv(System.M_tot);

%% Inititalizarion of reference filter
Delta = eye(6); %Critical damping
cutoff = 0.02;

Omega = [cutoff 0 0 0 0 0;       %cut off frequencies in 6 DOFs
         0 cutoff 0 0 0 0;
         0 0 cutoff 0 0 0;
         0 0 0 0.07 0 0;
         0 0 0 0 0.07 0;
         0 0 0 0 0 0.01];
     
nu_max=[1.03 0.67 0.62 0  0 1.05]';
nu_min=-nu_max*0.8;
nu_dot_max=[0.5 0.5 0.4 1 1 1]';
nu_dot_min = -nu_dot_max*0.8;

%% Initialization for controllers

% Nonlinear PID

Controller.Kp = diag([50 50 300]);
Controller.Kd = diag([300 300 1000]);
Controller.Ki = diag([1.5 1.5 2.5])*3;

%Sliding mode controller
Controller.Sliding.enableM=1;
Controller.Sliding.enableDL=1;
Controller.Sliding.enableDNL=1;
Controller.Sliding.enableC=1;
Controller.Sliding.enableG=0;

Controller.Sliding.integrator_limitation=[1.5 1.5 1.5 0 0 10*pi/180]';

Controller.Sliding.lambda=diag([0.1 0.1 0.1 0.1 0.1 0.1]);
Controller.Sliding.phi=1.5; 

Controller.Sliding.K_s=diag([100 100 100 0 0 60]);
Controller.Sliding.K_d=diag([960 160 320 0 0 160]); 

Observer_System = System;
sim_time = 1400;
time1 = 100;
step1 = [0 0 0 0 0 90*pi/180]';
time2 = 500;
step2 = [1.5 0 0 0 0 0]';
time3 = 900;
step3 = [0 1.5 0 0 0 0]';
time4 = 900;
step4 = [0 0 0 0 0 0]';
time5 = 900;
step5 = [0 0 0 0 0 0]';




