%%% Recursive thrust allocation parameters %%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% by Robert kajanus
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

% Recursive nullspace-based thrust allocation
%global ROV.Thrust.RTParam

% Calculating thruster limits
F1max = ROV.Thrust.k_fw * ROV.Thrust.n_max^2;
F2max = F1max;
F1min = ROV.Thrust.k_b * ROV.Thrust.n_max^2;
F2min = F1min;
%    F1min=F1max*0.7;
%    F2min = F1min;
F3max = ROV.Thrust.k_lat * ROV.Thrust.n_max^2;
F3min = F3max;
ROV.Thrust.RTParam.Tau_lim = [F1max F1min; F2max F2min; F3max F3min];
           
% Extracting B-matrix as submatrix of Tconfig
ROV.Thrust.RTParam.B = [ROV.Thrust.Tconfig(1:2,1:3); ROV.Thrust.Tconfig(4,1:3)];

% Extracting vectors from B-matrix
ROV.Thrust.RTParam.b_x_T = ROV.Thrust.RTParam.B(1,:); b_x = ROV.Thrust.RTParam.b_x_T';	% x vectors;   (1x3) , (3x1)
ROV.Thrust.RTParam.b_y_T = ROV.Thrust.RTParam.B(2,:); b_y = ROV.Thrust.RTParam.b_y_T';	% y vectors;   (1x3) , (3x1)
ROV.Thrust.RTParam.b_p_T = ROV.Thrust.RTParam.B(3,:);	b_p = ROV.Thrust.RTParam.b_p_T'; % psi vectors; (1x3) , (3x1)

%%% W is a tuning parameter => assigned to obtain maximum yaw moment %%%
% Weighting matrix, weighted for max Psi (max_Psi_comp / max_Psi_tot)
W = diag([abs((239*ROV.Thrust.RTParam.B(3,1)))/(151);...
          (239*ROV.Thrust.RTParam.B(3,2))/(151);... 
          (195*ROV.Thrust.RTParam.B(3,3))/(151)]);
      W=eye(3);%W(3,3) =1.4; %W(1,1)= 0.1; W(2,2)= 0.1;
%%% W is a tuning parameter => assigned to obtain maximum yaw moment %%%

%%%%%% 1st step: allocating yaw force, i.e. F_psi [N] %%%%%%
% Defining the pseudoinverse and the corresponding nullspace matrix
ROV.Thrust.RTParam.b_p_inv = inv(W) * b_p * inv(ROV.Thrust.RTParam.b_p_T * inv(W) * b_p);   % (3x1)
ROV.Thrust.RTParam.Q1 = eye(3) - ROV.Thrust.RTParam.b_p_inv * ROV.Thrust.RTParam.b_p_T;                % (3x3)

%%%%%% 2nd step: allocating sway force, i.e. F_y [N] %%%%%%
% Defining the pseudoinverse and the corresponding nullspace matrix
ROV.Thrust.RTParam.b_y_inv = inv(W) * ROV.Thrust.RTParam.Q1 * b_y * inv(ROV.Thrust.RTParam.b_y_T * ROV.Thrust.RTParam.Q1 * inv(W) * ROV.Thrust.RTParam.Q1 * b_y);
ROV.Thrust.RTParam.Q2 = eye(3) - ROV.Thrust.RTParam.b_y_inv * ROV.Thrust.RTParam.b_y_T * ROV.Thrust.RTParam.Q1;

%%%%%% 3rd step: allocating surge force, i.e. F_x [N] %%%%%%
% Defining the pseudoinverse
ROV.Thrust.RTParam.b_x_inv = inv(W) * ROV.Thrust.RTParam.Q2 * b_x * inv(ROV.Thrust.RTParam.b_x_T * ROV.Thrust.RTParam.Q2 * inv(W) * ROV.Thrust.RTParam.Q2 * b_x);

clear F1max F1min F2max F2min F3max F3min W b_p b_x b_y;